Package: lpdensity
Type: Package
Title: Local Polynomial Density Estimation and Inference
Date: 2026-05-23
Version: 3.0.1
Authors@R: c(
    person("Matias D.", "Cattaneo", email = "matias.d.cattaneo@gmail.com", role = c("aut", "cre")),
    person("Michael", "Jansson", email = "michael.jansson.berkeley@gmail.com", role = "aut"),
    person("Xinwei", "Ma", email = "xinweima.pku@gmail.com", role = "aut"))
Author: Matias D. Cattaneo [aut, cre],
  Michael Jansson [aut],
  Xinwei Ma [aut]
Maintainer: Matias D. Cattaneo <matias.d.cattaneo@gmail.com>
Description: Implements local polynomial distribution and density methods for point estimation, inference and bandwidth selection, documented in Cattaneo, Jansson and Ma (2020) <doi:10.1080/01621459.2019.1635480>, Cattaneo, Jansson and Ma (2022) <doi:10.18637/jss.v101.i02>, and Cattaneo, Jansson and Ma (2024) <doi:10.1016/j.jeconom.2021.01.006>. lpdensity() constructs local polynomial distribution and density estimators with robust bias-corrected inference, and lpbwdensity() implements data-driven bandwidth selection.
URL: https://nppackages.github.io/,
        https://github.com/nppackages/lpdensity
BugReports: https://github.com/nppackages/lpdensity/issues
Depends: R (>= 3.1)
License: GPL-3
Encoding: UTF-8
Imports: ggplot2, MASS
Suggests: testthat (>= 3.0.0)
RoxygenNote: 7.3.2
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-23 08:44:27 UTC; CodexSandboxOffline
Repository: CRAN
Date/Publication: 2026-05-23 11:20:02 UTC
Built: R 4.5.2; ; 2026-05-25 01:30:56 UTC; unix
