# Generated by roxygen2: do not edit by hand

S3method(coef,lpbwdensity)
S3method(coef,lpdensity)
S3method(confint,lpdensity)
S3method(plot,lpdensity)
S3method(print,lpbwdensity)
S3method(print,lpdensity)
S3method(summary,lpbwdensity)
S3method(summary,lpdensity)
S3method(vcov,lpdensity)
export(lpbwdensity)
export(lpdensity)
export(lpdensity.plot)
import(ggplot2)
importFrom(MASS,mvrnorm)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(stats,D)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,optimize)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
